import{n as e,t}from"./create-api-fetcher-BF8Rcwoh.js";const n=e({endpointKey:`ALL_GATEWAY_ENDPOINTS`,responseType:[]}),r=e({endpointKey:`ALL_GATEWAY_SECRETS`,responseType:[]}),i=e({endpointKey:`ALL_GATEWAY_MODELS`,responseType:[]}),a=t({endpointKey:`USER_GATEWAY_ENDPOINT_PERMISSIONS`,responseType:[]}),o=t({endpointKey:`USER_GATEWAY_SECRET_PERMISSIONS`,responseType:[]}),s=t({endpointKey:`USER_GATEWAY_MODEL_PERMISSIONS`,responseType:[]}),c=t({endpointKey:`USER_GATEWAY_ENDPOINT_PATTERN_PERMISSIONS`,responseType:[]}),l=t({endpointKey:`USER_GATEWAY_SECRET_PATTERN_PERMISSIONS`,responseType:[]}),u=t({endpointKey:`USER_GATEWAY_MODEL_PATTERN_PERMISSIONS`,responseType:[]}),d=t({endpointKey:`GROUP_GATEWAY_ENDPOINT_PERMISSIONS`,responseType:[]}),f=t({endpointKey:`GROUP_GATEWAY_SECRET_PERMISSIONS`,responseType:[]}),p=t({endpointKey:`GROUP_GATEWAY_MODEL_PERMISSIONS`,responseType:[]}),m=t({endpointKey:`GROUP_GATEWAY_ENDPOINT_PATTERN_PERMISSIONS`,responseType:[]}),h=t({endpointKey:`GROUP_GATEWAY_SECRET_PATTERN_PERMISSIONS`,responseType:[]}),g=t({endpointKey:`GROUP_GATEWAY_MODEL_PATTERN_PERMISSIONS`,responseType:[]}),_=t({endpointKey:`GATEWAY_ENDPOINT_USER_PERMISSIONS`,responseType:[]}),v=t({endpointKey:`GATEWAY_SECRET_USER_PERMISSIONS`,responseType:[]}),y=t({endpointKey:`GATEWAY_MODEL_USER_PERMISSIONS`,responseType:[]}),b=t({endpointKey:`GATEWAY_ENDPOINT_GROUP_PERMISSIONS`,responseType:[]}),x=t({endpointKey:`GATEWAY_SECRET_GROUP_PERMISSIONS`,responseType:[]}),S=t({endpointKey:`GATEWAY_MODEL_GROUP_PERMISSIONS`,responseType:[]});export{a as _,_ as a,l as b,x as c,d,g as f,c as g,f as h,b as i,v as l,h as m,i as n,S as o,p,r,y as s,n as t,m as u,u as v,o as x,s as y};