import{n as e,t}from"./create-api-fetcher-BF8Rcwoh.js";const n=e({endpointKey:`ALL_GROUPS`,responseType:[]}),r=e({endpointKey:`ALL_EXPERIMENTS`,responseType:[]}),i=e({endpointKey:`ALL_MODELS`,responseType:[]}),a=e({endpointKey:`ALL_PROMPTS`,responseType:[]}),o=t({endpointKey:`EXPERIMENT_USER_PERMISSIONS`,responseType:[]}),s=t({endpointKey:`USER_EXPERIMENT_PERMISSIONS`,responseType:[]}),c=t({endpointKey:`USER_MODEL_PERMISSIONS`,responseType:[]}),l=t({endpointKey:`USER_PROMPT_PERMISSIONS`,responseType:[]}),u=t({endpointKey:`USER_EXPERIMENT_PATTERN_PERMISSIONS`,responseType:[]}),d=t({endpointKey:`USER_MODEL_PATTERN_PERMISSIONS`,responseType:[]}),f=t({endpointKey:`USER_PROMPT_PATTERN_PERMISSIONS`,responseType:[]}),p=t({endpointKey:`MODEL_USER_PERMISSIONS`,responseType:[]}),m=t({endpointKey:`PROMPT_USER_PERMISSIONS`,responseType:[]}),h=t({endpointKey:`GROUP_EXPERIMENT_PERMISSIONS`,responseType:[]}),g=t({endpointKey:`GROUP_MODEL_PERMISSIONS`,responseType:[]}),_=t({endpointKey:`GROUP_PROMPT_PERMISSIONS`,responseType:[]}),v=t({endpointKey:`EXPERIMENT_GROUP_PERMISSIONS`,responseType:[]}),y=t({endpointKey:`MODEL_GROUP_PERMISSIONS`,responseType:[]}),b=t({endpointKey:`PROMPT_GROUP_PERMISSIONS`,responseType:[]}),x=t({endpointKey:`GROUP_EXPERIMENT_PATTERN_PERMISSIONS`,responseType:[]}),S=t({endpointKey:`GROUP_MODEL_PATTERN_PERMISSIONS`,responseType:[]}),C=t({endpointKey:`GROUP_PROMPT_PATTERN_PERMISSIONS`,responseType:[]});export{c as S,u as _,v as a,f as b,h as c,_ as d,g as f,m as g,b as h,a as i,S as l,p as m,n,o,y as p,i as r,x as s,r as t,C as u,s as v,l as x,d as y};