import{o as e,r as t}from"./button-DJ30HgDO.js";import"./use-runtime-config-CkwqDzno.js";import"./use-api-CtLXpHJw.js";import"./use-user-D7emtooi.js";import"./user-service-Dl9A7L06.js";import"./toast-context-val-Z-lwvYkJ.js";import"./gateway-service-BwyflEJR.js";import"./use-all-gateway-endpoints-CoteZrGd.js";import"./page-status-CNIWXN7q.js";import"./page-container-DN56Bi3o.js";import"./icon-button-DFhNpcsD.js";import"./use-toast-BWmLhWRD.js";import"./input-BbiN5lj4.js";import"./grant-permission-modal-CddXPWUg.js";import"./entity-service-KdPsD3WQ.js";import"./use-all-gateway-models-B4V5vz8a.js";import"./use-all-gateway-secrets-DbEZsODo.js";import"./use-all-experiments-C6QGQbcz.js";import"./switch-44nkuCsc.js";import"./use-user-gateway-model-permissions-C1ILLuXb.js";import{t as n}from"./shared-permissions-page-DdsT3t3f.js";import"./use-all-models-oH52dpvM.js";import"./use-all-prompts-DI2n7rek.js";var r=e(t(),1);function i({type:e}){return(0,r.jsx)(n,{type:e,baseRoute:`/service-accounts`,entityKind:`user`})}export{i as default};