import{i as e,o as t,r as n,t as r}from"./button-DJ30HgDO.js";import{o as i}from"./chunk-EPOLDU6W-BGm2xlSj.js";import{m as a}from"./free-solid-svg-icons-CttZC1b4.js";import{t as o}from"./string-utils-CqCEOCqZ.js";e();var s=t(n(),1);function c({entityId:e,route:t,buttonText:n,suffix:c=``}){let l=i(),u=`/${t.replace(/^\/+/,``)}/${o(e)}${c}`;return(0,s.jsx)(r,{onClick:e=>{e.stopPropagation(),l(u)},icon:a,className:`gap-1`,children:n})}export{c as t};