import{i as e,o as t}from"./button-DJ30HgDO.js";var n=t(e(),1);const r=(0,n.createContext)(null);function i(){let e=(0,n.use)(r);if(!e)throw Error(`useRuntimeConfig must be used within a RuntimeConfigProvider`);return e}export{i as n,r as t};