import{o as e,r as t,t as n}from"./button-DJ30HgDO.js";import{o as r}from"./chunk-EPOLDU6W-BGm2xlSj.js";import{t as i}from"./page-container-DN56Bi3o.js";var a=e(t(),1);const o=()=>{let e=r();return(0,a.jsx)(i,{title:`Page Not Found`,children:(0,a.jsxs)(`div`,{className:`flex flex-col items-center justify-center h-[50vh] space-y-6 text-center`,children:[(0,a.jsxs)(`div`,{className:`space-y-2`,children:[(0,a.jsx)(`h1`,{className:`text-4xl font-bold text-text-primary dark:text-text-primary-dark`,children:`404`}),(0,a.jsx)(`p`,{className:`text-xl text-text-secondary dark:text-text-secondary-dark`,children:`Oops! The page you are looking for does not exist.`})]}),(0,a.jsx)(n,{onClick:()=>{e(`/user`)},variant:`primary`,children:`Go to My Profile`})]})})};var s=o;export{o as NotFoundPage,s as default};