import{t as e}from"./use-api-CtLXpHJw.js";import{t}from"./gateway-service-BwyflEJR.js";function n(){let{data:n,isLoading:r,error:i,refetch:a}=e(t);return{allGatewayEndpoints:n,isLoading:r,error:i,refresh:a}}export{n as t};